/*
 @C:\Web_505\Sql\End_Conv\Oracle\5104_DFSSTPO_Calcula_AliqImposto.SQL

 Descrio:
 Procedure que retorna a alquota dos impostos

 Parmetros:
 P_ENTRSAID     - Identificador da operao ("E" - Entrada/"S" - Sada)
 P_EMPRESA      - Cdigo da empresa
 P_CDARVPROD    - Cdigo de rvore do produto
 P_INSCESTA     - Inscrio estadual do lanamento
 P_CDTIPOOPER   - Tipo de Operao do lanamento
 P_FILIAL       - Filial do lanamento
 P_CDFILIEMIT   - Filial de origem do lanamento
 P_CDCLIENTE    - Cdigo do destinatrio do lanamento (CDFILIRECE, CDCLIENTE, CDFORNECED)
 P_CDIMPOSTO    - Cdigo do Imposto desejado
 P_VRTOTAITEM   - Valor total do item
 P_INIVALID     - Data de entrada/sada do lanamento
 P_VRALIQUOTA   - Parmetro que retorno o valor da alquota

 Observao:
 Esta procedure deve ser chamada uma vez por imposto.

 Exemplo da utilizao:
 "CALCULA_ALIQIMPOSTO('S', '01', '1010100200', 'ISENTO', '02', '9999', '0002', '00',10,'01/04/2009');"

 raise_application_error(-20501, '' , True);
*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
            WHERE O.NAME = 'CONSULTA_ALIQIMPOSTO'
              AND TYPE = 'P'
              AND U.UID = O.UID
              AND U.NAME = USER)
DROP PROCEDURE CONSULTA_ALIQIMPOSTO
GO

CREATE PROCEDURE CONSULTA_ALIQIMPOSTO @P_ENTRSAID   VARCHAR(01),
                                      @P_EMPRESA    VARCHAR(02),
                                      @P_CDARVPROD  VARCHAR(20),
                                      @P_INSCESTA   VARCHAR(16),
                                      @P_CDTIPOOPER VARCHAR(02),
                                      @P_FILIAL     VARCHAR(04),
                                      @P_CDFILIEMIT VARCHAR(04),
                                      @P_CDCLIENTE  VARCHAR(21),
                                      @P_CDIMPOSTO  VARCHAR(02),
                                      @P_VRTOTAITEM FLOAT,
                                      @P_INIVALID   DATETIME,
                                      @P_VRALIQUOTA FLOAT OUTPUT
AS
   DECLARE @V_IDORIGDESTNF  VARCHAR(01);
   DECLARE @V_CDGRUPOFISC   VARCHAR(03);
   DECLARE @V_PAISORIG      VARCHAR(04);
   DECLARE @V_ESTAORIG      VARCHAR(02);
   DECLARE @V_MUNIORIG      VARCHAR(04);
   DECLARE @V_PAISDEST      VARCHAR(04);
   DECLARE @V_ESTADEST      VARCHAR(02);
   DECLARE @V_MUNIDEST      VARCHAR(04);
   DECLARE @V_IDENDCFOP     VARCHAR(01);
   DECLARE @V_IDENDEMIS     VARCHAR(01);
   DECLARE @V_IDPRODNOTA    VARCHAR(01);
   DECLARE @V_IDUTMUNICIPIO VARCHAR(01);
   DECLARE @V_TAXAALIQUOTA  FLOAT;
   DECLARE @V_ALIQUTMP      FLOAT;
   DECLARE @V_PREDBCTMP     FLOAT;
   DECLARE @V_IDINTEREST    VARCHAR(01);
   DECLARE @V_CDEMPRESA     VARCHAR(02);
   DECLARE @V_CDINSCESTA    VARCHAR(16);

   /*VARIVEIS UTILIZADA NO CURSOR C_INCIDENCIA*/
   DECLARE @V_INCIDENCIA_CDIMPOTEMP   VARCHAR(02);
   DECLARE @V_INCIDENCIA_CDFISCTEMP   VARCHAR(01);
   DECLARE @V_INCIDENCIA_IDTRATIMPTMP VARCHAR(01);
   DECLARE @V_INCIDENCIA_IDUTMUNIORIG VARCHAR(01);
   DECLARE @V_INCIDENCIA_IDCONSISTENT VARCHAR(01);
   DECLARE @V_INCIDENCIA_IDIMPORECUP  VARCHAR(01);

BEGIN
   SELECT @V_IDENDCFOP = ISNULL(MAX(IDENDCFOPFIL),'I')
     FROM FILIAL
    WHERE (CDFILIAL = @P_CDFILIEMIT);

   IF @V_IDENDCFOP = 'I' BEGIN
      SELECT @V_PAISORIG = MAX(CDPAIS),
             @V_ESTAORIG = MAX(SGESTADO),
             @V_MUNIORIG = MAX(CDMUNICIPIO)
        FROM INSCRESTAD
       WHERE CDEMPRESA  = @P_EMPRESA
         AND CDINSCESTA = @P_INSCESTA;
   END
   ELSE BEGIN
      SELECT @V_PAISORIG = MAX(CDPAIS),
             @V_ESTAORIG = MAX(SGESTADO),
             @V_MUNIORIG = MAX(CDMUNICIPIO)
        FROM ENDEFILI
       WHERE (CDFILIAL     = @P_CDFILIEMIT) AND
             (IDTPENDEFILI = @V_IDENDCFOP);
   END;

   SELECT @V_IDORIGDESTNF = MAX(IDORIGDESTOP)
   FROM TIPOOPERAC
   WHERE CDTIPOOPER   = @P_CDTIPOOPER
     AND IDENTRSAIDOP = @P_ENTRSAID;

   IF @V_IDORIGDESTNF IS NULL BEGIN
      SET @V_IDORIGDESTNF = 'C';
   END;

   IF @V_IDORIGDESTNF = 'L' BEGIN
      SELECT @V_IDENDEMIS  = ISNULL(MAX(IDENDEMISFIL),'I'),
             @V_CDEMPRESA  = MAX(CDEMPRESA),
             @V_CDINSCESTA = MAX(CDINSCESTA)
        FROM FILIAL
       WHERE (CDFILIAL = @P_CDCLIENTE);

      IF @V_IDENDEMIS = 'I' BEGIN
          SELECT @V_PAISDEST = MAX(CDPAIS),
                 @V_ESTADEST = MAX(SGESTADO),
                 @V_MUNIDEST = MAX(CDMUNICIPIO)
            FROM INSCRESTAD
           WHERE CDEMPRESA  = @V_CDEMPRESA
             AND CDINSCESTA = @V_CDINSCESTA;
      END
      ELSE BEGIN
       SELECT @V_PAISDEST = MAX(CDPAIS),
              @V_ESTADEST = MAX(SGESTADO),
              @V_MUNIDEST = MAX(CDMUNICIPIO)
         FROM ENDEFILI
        WHERE CDFILIAL     = @P_CDCLIENTE
          AND IDTPENDEFILI = @V_IDENDEMIS;
      END;
   END
   ELSE BEGIN
      IF @V_IDORIGDESTNF = 'F' BEGIN
         SELECT @V_IDENDEMIS = ISNULL(MAX(IDENDEMISFOR),'P')
           FROM FORNECEDOR
          WHERE (CDFORNECED = @P_CDCLIENTE);

         SELECT @V_PAISDEST = MAX(CDPAIS),
                @V_ESTADEST = MAX(SGESTADO),
                @V_MUNIDEST = MAX(CDMUNICIPIO)
           FROM ENDEFORN
          WHERE (CDFORNECED   = @P_CDCLIENTE)
            AND (IDTPENDEFORN = @V_IDENDEMIS);
      END
      ELSE BEGIN
         SELECT @V_IDENDEMIS = ISNULL(MAX(IDENDEMISCLI),'P')
           FROM CLIENTE
          WHERE (CDCLIENTE = @P_CDCLIENTE);

         SELECT @V_PAISDEST = MAX(CDPAIS),
                @V_ESTADEST = MAX(SGESTADO),
                @V_MUNIDEST = MAX(CDMUNICIPIO)
           FROM ENDECLIE
          WHERE (CDCLIENTE    = @P_CDCLIENTE)
            AND (IDTPENDECLIE = @V_IDENDEMIS);
      END;
   END;

   SET @V_IDINTEREST = 'N';

   IF (@V_PAISDEST = @V_PAISORIG) AND (@V_ESTADEST <> @V_ESTAORIG) BEGIN
      SET @V_IDINTEREST = 'S';
   END;

   EXEC CONSULTA_GRUPOFISCAL @V_IDORIGDESTNF, @P_CDCLIENTE, '', @P_ENTRSAID, @P_CDTIPOOPER, @V_CDGRUPOFISC OUTPUT;

   EXEC CONSULTA_INCIDENCIA @P_ENTRSAID, @P_EMPRESA, @P_CDARVPROD, @P_INSCESTA, @P_CDTIPOOPER, @P_FILIAL, '0000000000', @V_CDGRUPOFISC, NULL, @V_IDINTEREST OUTPUT;

   SELECT @V_IDPRODNOTA = MAX(IDPRODNOTA)
     FROM IMPOSTO
    WHERE CDIMPOSTO = @P_CDIMPOSTO;

   DECLARE C_INCIDENCIA CURSOR FOR SELECT DISTINCT (CDIMPOTEMP) AS CDIMPOTEMP, CDFISCTEMP,
                                          IDTRATIMPTMP, IDUTMUNIORIG, IDCONSISTENT, IDIMPORECUP
                                     FROM INCITEMP IT, IMPOSTO IP
                                    WHERE (IT.NRINCITEMP = @P_FILIAL+'0000000000')
                                      AND (IP.CDIMPOSTO  = IT.CDIMPOTEMP)
                                      AND (IP.IDPRODNOTA = @V_IDPRODNOTA)
                                      AND (IP.CDIMPOSTO  = @P_CDIMPOSTO)
                                   ORDER BY CDIMPOTEMP;
   OPEN C_INCIDENCIA;
   FETCH NEXT FROM C_INCIDENCIA INTO @V_INCIDENCIA_CDIMPOTEMP, @V_INCIDENCIA_CDFISCTEMP, @V_INCIDENCIA_IDTRATIMPTMP,
                                     @V_INCIDENCIA_IDUTMUNIORIG, @V_INCIDENCIA_IDCONSISTENT, @V_INCIDENCIA_IDIMPORECUP;

   WHILE (@@FETCH_STATUS = 0) BEGIN
      SELECT @V_IDUTMUNICIPIO = ISNULL(MAX(IDUTMUNIORIG), 'N')
        FROM IMPOSTO
       WHERE CDIMPOSTO = @V_INCIDENCIA_CDIMPOTEMP;

      IF ((@V_IDUTMUNICIPIO = 'O') OR (@V_IDUTMUNICIPIO = 'N')) BEGIN
         EXEC CONSULTA_ALIQUOTA @V_INCIDENCIA_CDIMPOTEMP, @P_ENTRSAID, @P_EMPRESA, @P_CDARVPROD, @P_INSCESTA, @V_PAISORIG,
                                @V_ESTAORIG, @V_PAISDEST, @V_ESTADEST, @V_MUNIORIG, @P_VRTOTAITEM, @P_INIVALID, @P_FILIAL,
                                '0000000000', @V_CDGRUPOFISC, @P_CDTIPOOPER;
      END
      ELSE BEGIN
         EXEC CONSULTA_ALIQUOTA @V_INCIDENCIA_CDIMPOTEMP, @P_ENTRSAID, @P_EMPRESA, @P_CDARVPROD, @P_INSCESTA, @V_PAISORIG,
                                @V_ESTAORIG, @V_PAISDEST, @V_ESTADEST, @V_MUNIDEST, @P_VRTOTAITEM, @P_INIVALID, @P_FILIAL,
                                '0000000000', @V_CDGRUPOFISC, @P_CDTIPOOPER;
      END;

      SELECT @V_ALIQUTMP  = ISNULL(MAX(VRALIQUTMP),0),
             @V_PREDBCTMP = ISNULL(MAX(VRPREDBCTMP),0)
      FROM ALIQTEMP
      WHERE (NRALIQTEMP = @P_FILIAL+'0000000000');

      IF ((@V_INCIDENCIA_IDIMPORECUP = 'S') AND (@V_INCIDENCIA_CDFISCTEMP = '1')) OR (@V_INCIDENCIA_IDIMPORECUP <> 'S') BEGIN
         SET @V_TAXAALIQUOTA = ((100-@V_PREDBCTMP)/100) * (@V_ALIQUTMP/100);
      END
      ELSE BEGIN
         SET @V_TAXAALIQUOTA = 0;
      END;

      DELETE FROM ALIQTEMP
      WHERE NRALIQTEMP = @P_FILIAL+'0000000000';

      FETCH NEXT FROM C_INCIDENCIA INTO @V_INCIDENCIA_CDIMPOTEMP, @V_INCIDENCIA_CDFISCTEMP, @V_INCIDENCIA_IDTRATIMPTMP,
                                        @V_INCIDENCIA_IDUTMUNIORIG, @V_INCIDENCIA_IDCONSISTENT, @V_INCIDENCIA_IDIMPORECUP;
   END;
   CLOSE C_INCIDENCIA;
   DEALLOCATE C_INCIDENCIA;

   DELETE INCITEMP
   WHERE NRINCITEMP = @P_FILIAL+'0000000000';
   SET @P_VRALIQUOTA = ISNULL(@V_TAXAALIQUOTA,0);
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'CALCULA_ALIQIMPOSTO','5.50.xxx';
GO
